@@ -2,8 +2,8 @@
 #
 # refineqtl.R
 #
-# copyright (c) 2006-2010, Karl W. Broman
-# last modified Jul, 2010
+# copyright (c) 2006-2011, Karl W. Broman
+# last modified Feb, 2011
 # first written Jun, 2006
 #
 #     This program is free software; you can redistribute it and/or
@@ -183,6 +183,15 @@ function(cross, pheno.col=1, qtl, chr, pos, qtl.name, covar=NULL, formula,
       formula <- paste(formula, "+", paste(colnames(covar), collapse="+"))
     formula <- as.formula(formula)
   }
+
+  # drop covariates that are not in the formula
+  if(!is.null(covar)) {
+    theterms <- rownames(attr(terms(formula), "factors"))
+    m <- match(colnames(covar), theterms)
+    if(all(is.na(m))) covar <- NULL
+    else covar <- covar[,!is.na(m),drop=FALSE]
+  }
+
   formula <- checkformula(formula, qtl$altname, colnames(covar))
 
   # identify which QTL are in the model formula
@@ -356,9 +365,7 @@ function(cross, pheno.col=1, qtl, chr, pos, qtl.name, covar=NULL, formula,
       lastout[[i]] <- data.frame(chr=chr, pos=pos, lod=as.numeric(lastout[[i]]))
     }
     names(lastout) <- qtl$name[tovary]
-  }
 
-  if(keeplodprofile) {
     # make the profiles scanone objects
     for(i in seq(along=lastout)) {
       class(lastout[[i]]) <- c("scanone", "data.frame")
@@ -368,12 +375,14 @@ function(cross, pheno.col=1, qtl, chr, pos, qtl.name, covar=NULL, formula,
       else
         detailedmap <- attr(cross$geno[[thechr]]$prob,"map")
     
+      if(is.matrix(detailedmap)) detailedmap <- detailedmap[1,]
+
       r <- range(lastout[[i]][,2])+c(-1e-5, 1e-5)
       rn <- names(detailedmap)[detailedmap>=r[1] & detailedmap<=r[2]]
       o <- grep("^loc-*[0-9]+",rn)
       if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
         rn[o] <- paste("c",thechr,".",rn[o],sep="")
-      if(length(rn) != nrow(lastout[[i]])) return(list(lastout[[i]], rn, detailedmap))
+#      if(length(rn) != nrow(lastout[[i]])) return(list(lastout[[i]], rn, detailedmap))
       if(length(rn) == nrow(lastout[[i]])) rownames(lastout[[i]]) <- rn
     }
 
