@@ -3,7 +3,7 @@
 # ripple.R
 #
 # copyright (c) 2001-2010, Karl W Broman
-# last modified Jul, 2010
+# last modified Nov, 2010
 # first written Oct, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -124,6 +124,7 @@ function(cross, chr, window=4, method=c("countxo","likelihood"),
       clusterStopped <- FALSE
       on.exit(if(!clusterStopped) stopCluster(cl))
       if(verbose) cat("   Running in", n.cluster, "clusters\n")
+      clusterEvalQ(cl, require(qtl, quietly=TRUE))
 
       whclust <- sort(rep(1:n.cluster, ceiling(n.orders/n.cluster))[1:n.orders])
       order.split <- vector("list", n.cluster)
@@ -139,7 +140,8 @@ function(cross, chr, window=4, method=c("countxo","likelihood"),
       for(i in 1:n.orders) {
         if(verbose && (i %/% print.by)*print.by == i) cat("    --Order", i, "\n")
         temcross$geno[[1]]$data <- cross$geno[[1]]$data[,orders[i,]]
-        newmap <- est.map(temcross,error.prob,map.function,m=0,p=0,maxit,tol,sex.sp,FALSE)
+        newmap <- est.map(temcross, error.prob=error.prob, map.function=map.function,
+                          m=0, p=0, maxit=maxit, tol=tol, sex.sp=sex.sp, verbose=FALSE)
         loglik[i] <- attr(newmap[[1]],"loglik")
         chrlen[i] <- diff(range(newmap[[1]]))
       }
@@ -183,6 +185,7 @@ function(cross, chr, window=4, method=c("countxo","likelihood"),
       clusterStopped <- FALSE
       on.exit(if(!clusterStopped) stopCluster(cl))
       if(verbose) cat("   Running in", n.cluster, "clusters\n")
+      clusterEvalQ(cl, require(qtl, quietly=TRUE))
 
       whclust <- sort(rep(1:n.cluster, ceiling(n.orders/n.cluster))[1:n.orders])
       order.split <- vector("list", n.cluster)
@@ -242,7 +245,8 @@ function(orders, cross, error.prob, map.function, maxit, tol, sex.sp)
   for(i in 1:n.orders) {
     
     temcross$geno[[1]]$data <- cross$geno[[1]]$data[,orders[i,]]
-    newmap <- est.map(temcross,error.prob,map.function,m=0,p=0,maxit,tol,sex.sp,FALSE)
+    newmap <- est.map(temcross, error.prob=error.prob, map.function=map.function,
+                      m=0, p=0, maxit=maxit, tol=tol, sex.sp=sex.sp, verbose=FALSE)
     loglik[i] <- attr(newmap[[1]],"loglik")
     chrlen[i] <- diff(range(newmap[[1]]))
   }
