@@ -3,7 +3,7 @@
 # summary.scanone.R
 #
 # copyright (c) 2001-2010, Karl W Broman
-# last modified Jul, 2010
+# last modified Nov, 2010
 # first written Sep, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -23,7 +23,7 @@
 #           max.scanone, c.scanone, subset.scanone,
 #           summary.scanoneperm, print.summary.scanoneperm
 #           c.scanoneperm, rbind.scanoneperm, cbind.scanoneperm
-#           grab.arg.names, subset.scantwoperm
+#           grab.arg.names, subset.scanoneperm, [.scanoneperm
 #
 ######################################################################
 
@@ -31,8 +31,9 @@
 # summarize scanone results 
 ##################################################################
 summary.scanone <-
-function(object, threshold, format=c("onepheno", "allpheno", "allpeaks"),
-         perms, alpha, lodcolumn=1, pvalues=FALSE, df=FALSE, ...)
+function(object, threshold, format=c("onepheno", "allpheno", "allpeaks", "tabByCol", "tabByChr"),
+         perms, alpha, lodcolumn=1, pvalues=FALSE, df=FALSE, 
+         ci.function=c("lodint", "bayesint"), ...)
 {
   if(!any(class(object) == "scanone"))
     stop("Input should have class \"scanone\".")
@@ -41,7 +42,7 @@ function(object, threshold, format=c("onepheno", "allpheno", "allpeaks"),
   ncol.object <- ncol(object)-2
   cn.object <- colnames(object)[-(1:2)]
 
-  if(ncol.object==1 && format != "onepheno") {
+  if(ncol.object==1 && (format == "allpeaks" || format == "allpheno")) {
     warning("With just one LOD column, format=\"onepheno\" used.")
     format <- "onepheno"
   }
@@ -229,7 +230,7 @@ function(object, threshold, format=c("onepheno", "allpheno", "allpeaks"),
     result <- object[wh,]
   }  # end of format=="allpheno"
 
-  else { # format=="allpeaks"
+  else if(format=="allpeaks") {
     # pull out max on each chromosome
     wh <- vector("list", ncol.object)
 
@@ -297,76 +298,224 @@ function(object, threshold, format=c("onepheno", "allpheno", "allpeaks"),
       result[,(1:ncol.object)*2] <- pos
       result[,(1:ncol.object)*2+1] <- lod
     }
-  }    
-
-  if(pvalues && nrow(result) > 0) { # get p-values and add to the results
-    rn <- rownames(result)
-
-    if("xchr" %in% names(attributes(perms))) {
-      xchr <- attr(perms, "xchr")
-      xchr <- names(xchr)[xchr]
-      xchr <- as.character(result[,1]) %in% xchr
-      L <- attr(perms, "L")
-      Lt <- sum(L)
-      
-      pval <- vector("list", ncol.object)
-      for(i in 1:ncol.object) {
-        if(format=="allpeaks") thecol <- i*2+1
-        else thecol <- i+2
-        
-        pval[[i]] <- rep(0, length(xchr))
-        if(any(xchr))
-          pval[[i]][xchr] <- sapply(result[xchr,thecol], function(a, b, rat)
-                                    1-mean(b < a)^rat, perms$X[,i], Lt/L[2])
-        if(any(!xchr))
-          pval[[i]][!xchr] <- sapply(result[!xchr,thecol], function(a, b, rat)
-                                     1-mean(b < a)^rat, perms$A[,i], Lt/L[1])
-      }
-    }
-    else {
-      pval <- vector("list", ncol.object)
-      for(i in 1:ncol.object) {
-        if(format=="allpeaks") thecol <- i*2+1
-        else thecol <- i+2
-        
-        pval[[i]] <- sapply(result[,thecol], function(a, b) mean(b >= a), perms[,i])
-      }
-    }
-    
-    if(format == "allpeaks") {
-      temp <- as.data.frame(matrix(nrow=nrow(result), ncol=ncol.object*3+1))
-    
-      names(temp)[1] <- names(result)[1]
-      temp[,1] <- result[,1]
-
-      for(i in 1:ncol.object) {
-        names(temp)[i*3+(-1:1)] <- c(names(result)[i*2+(0:1)], "pval")
-        temp[,i*3-1:0] <- result[,i*2+(0:1)]
-        temp[,i*3+1] <- pval[[i]]
-      }
-    }
-    else {
-      temp <- as.data.frame(matrix(nrow=nrow(result), ncol=ncol.object*2+2))
-    
-      names(temp)[1:2] <- names(result)[1:2]
-      temp[,1:2] <- result[,1:2]
-      for(i in 1:ncol.object) {
-        names(temp)[i*2+1:2] <- c(names(result)[i+2], "pval")
-        temp[,i*2+1] <- result[,i+2]
-        temp[,i*2+2] <- pval[[i]]
-      }
-    }
-    result <- temp
-    rownames(result) <- rn
   }
-  
+  else { # format=="tabByChr" or =="tabByCol"
+    result <- vector("list", ncol.object)
+    names(result) <- names(object)[-(1:2)]
+
+    # pull out max on each chromosome
+    wh <- vector("list", ncol.object)
+
+    for(lodcolumn in (1:ncol.object)+2) {
+      for(i in unique(chr)) {
+        if(any(!is.na(object[chr==i,lodcolumn]))) {
+          mx <- max(object[chr==i,lodcolumn],na.rm=TRUE)
+          temp <- which(chr==i & object[,lodcolumn]==mx)
+          if(length(temp)>1) temp <- sample(temp, 1)
+          wh[[lodcolumn-2]] <- c(wh[[lodcolumn-2]], temp)
+        }
+        else 
+          wh[[lodcolumn-2]] <- c(wh, NA)
+      }
+    }
+
+    pos <- sapply(wh, function(a,b) b[a], object[,2])
+    if(!is.matrix(pos)) pos <- as.matrix(pos)
+
+    lod <- pos
+
+    for(i in 1:ncol(pos))
+      lod[,i] <- object[wh[[i]],i+2]
+    thechr <- as.character(unique(object[,1]))
+
+    for(i in 1:ncol.object) 
+      result[[i]] <- object[wh[[i]],c(1,2,i+2)]
+  }
+
+
+  if(pvalues) {
+    if(format != "tabByCol" && format != "tabByChr") {
+      if(nrow(result) > 0) { # get p-values and add to the results
+        rn <- rownames(result)
+
+        if("xchr" %in% names(attributes(perms))) {
+          xchr <- attr(perms, "xchr")
+          xchr <- names(xchr)[xchr]
+          xchr <- as.character(result[,1]) %in% xchr
+          L <- attr(perms, "L")
+          Lt <- sum(L)
+      
+          if(format=="allpeaks")
+            thecol <- (1:ncol.object)*2+1
+          else thecol <- (1:ncol.object)+2
+
+          if(any(xchr)) {
+            tempX <- calcPermPval(result[xchr,thecol,drop=FALSE], perms$X)
+            tempX <- as.data.frame(1-(1-tempX)^(Lt/L[2]))
+          }
+          else tempX <- NULL
+          if(any(!xchr)) {
+            tempA <- calcPermPval(result[!xchr,thecol,drop=FALSE], perms$A)
+            tempA <- as.data.frame(1-(1-tempA)^(Lt/L[1]))
+          }
+          else tempA <- NULL
+          pval <- rbind(tempA, tempX)
+          if(any(xchr)) pval[xchr,] <- tempX
+          if(any(!xchr)) pval[!xchr,] <- tempA
+        }
+        else {
+          if(format=="allpeaks") thecol <- (1:ncol.object)*2+1
+          else thecol <- (1:ncol.object)+2
+
+          pval <- as.data.frame(calcPermPval(result[,thecol,drop=FALSE], perms))
+        }
+
+        if(format == "allpeaks") {
+          temp <- as.data.frame(matrix(nrow=nrow(result), ncol=ncol.object*3+1))
+    
+          names(temp)[1] <- names(result)[1]
+          temp[,1] <- result[,1]
+
+          for(i in 1:ncol.object) {
+            names(temp)[i*3+(-1:1)] <- c(names(result)[i*2+(0:1)], "pval")
+            temp[,i*3-1:0] <- result[,i*2+(0:1)]
+            temp[,i*3+1] <- pval[[i]]
+          }
+        }
+        else if(format != "tabByCol" && format != "tabByChr") {
+          temp <- as.data.frame(matrix(nrow=nrow(result), ncol=ncol.object*2+2))
+    
+          names(temp)[1:2] <- names(result)[1:2]
+          temp[,1:2] <- result[,1:2]
+          for(i in 1:ncol.object) {
+            names(temp)[i*2+1:2] <- c(names(result)[i+2], "pval")
+            temp[,i*2+1] <- result[,i+2]
+            temp[,i*2+2] <- pval[[i]]
+          }
+        }
+        result <- temp
+        rownames(result) <- rn
+
+      }
+    }
+    else { # format=="tabByCol" || format=="tabByChr"
+      peaks <- as.data.frame(lapply(result, function(a) a[,3]))
+
+      if("xchr" %in% names(attributes(perms))) {
+        xchr <- attr(perms, "xchr")
+        xchr <- names(xchr)[xchr]
+        xchr <- as.character(result[[1]][,1]) %in% xchr
+        L <- attr(perms, "L")
+        Lt <- sum(L)
+      
+        if(any(xchr)) {
+          tempX <- as.data.frame(calcPermPval(peaks[xchr,,drop=FALSE], perms$X))
+          tempX <- 1-(1-tempX)^(Lt/L[2])
+        }
+        else tempX <- NULL
+        if(any(!xchr)) {
+          tempA <- as.data.frame(calcPermPval(peaks[!xchr,,drop=FALSE], perms$A))
+          tempA <- 1-(1-tempA)^(Lt/L[1])
+        }
+        else tempA <- NULL
+
+        pval <- rbind(tempA, tempX)
+        if(any(xchr)) pval[xchr,] <- tempX
+        if(any(!xchr)) pval[!xchr,] <- tempA
+      }
+      else 
+        pval <- as.data.frame(calcPermPval(peaks, perms))
+      
+      for(i in seq(along=result))
+        result[[i]] <- cbind(as.data.frame(result[[i]]), pval=pval[,i])
+    }
+  }
+
+  if(format=="tabByCol" || format=="tabByChr") { 
+
+    # drop insignificant peaks
+    if(!missing(threshold)) { # rows with at least one LOD > threshold
+      for(i in seq(along=result))
+        result[[i]] <- result[[i]][lod[,i] > threshold,,drop=FALSE]
+    }
+    else if(!missing(alpha)) {
+      keep <- NULL
+      thr <- summary(perms, alpha)
+
+      if("xchr" %in% names(attributes(perms))) {
+        xchr <- attr(perms, "xchr")
+        xchr <- names(xchr)[xchr]
+        xchr <- thechr %in% xchr
+
+        for(i in seq(along=result))
+          result[[i]] <- result[[i]][(lod[,i] > thr$A[i] & !xchr) |
+                                     (lod[,i] > thr$X[i] & xchr), , drop=FALSE]
+      }
+      else {
+        for(i in seq(along=result))
+          result[[i]] <- result[[i]][lod[,i] > thr[i],,drop=FALSE]
+      }
+    }
+
+    # add intervals
+    ci.function <- match.arg(ci.function)
+    if(ci.function=="lodint") cif <- lodint
+    else cif <- bayesint
+
+    for(i in seq(along=result)) {
+      if(nrow(result[[i]]) == 0) next
+
+      lo <- hi <- rep(NA, nrow(result[[i]]))
+      for(j in 1:nrow(result[[i]])) {
+        temp <- cif(object, chr=as.character(result[[i]][j,1]), lodcolumn=i, ...)
+        lo[j] <- temp[1,2]
+        hi[j] <- temp[nrow(temp),2]
+      }
+      result[[i]] <- cbind(as.data.frame(result[[i]]), ci.low=lo, ci.high=hi)
+      colnames(result[[i]])[3] <- "lod"
+    }
+
+    if(format=="tabByChr" && length(result)==1)
+      format <- "tabByCol"     # no need to do by chr in this case
+
+    if(format=="tabByChr") {
+      temp <- vector("list", length(thechr))
+      names(temp) <- thechr
+
+      for(i in seq(along=result)) {
+        if(nrow(result[[i]])==0) next
+        rownames(result[[i]]) <- paste(names(result)[i], rownames(result[[i]]), sep=" : ")
+        for(j in 1:nrow(result[[i]])) {
+          thischr <- match(result[[i]][j,1], thechr)
+          if(length(temp[[thischr]])==0)
+            temp[[thischr]] <- result[[i]][j,,drop=FALSE]
+          else 
+            temp[[thischr]] <- rbind(temp[[thischr]], result[[i]][j,,drop=FALSE])
+        }
+      }
+      result <- temp
+    }
+
+    # move CI to before the lod score
+    for(i in seq(along=result)) {
+      if(is.null(result[[i]]) || nrow(result[[i]])==0) next
+      nc <- ncol(result[[i]])
+      result[[i]] <- result[[i]][,c(1,2,nc-1,nc,3:(nc-2)),drop=FALSE]
+    }
+
+    attr(result, "tab") <- format
+  }
+
   if(df && "df" %in% names(attributes(object)))
     attr(result, "df") <- attr(object, "df")
   if(!df) attr(result, "df") <- NULL
 
   if(format=="allpeaks") rownames(result) <- as.character(result$chr)
 
-  class(result) <- c("summary.scanone", "data.frame")
+  if(format=="tabByCol" || format=="tabByChr")
+    class(result) <- c("summary.scanone", "list")
+  else
+    class(result) <- c("summary.scanone", "data.frame")
   result
 }
 
@@ -374,7 +523,9 @@ function(object, threshold, format=c("onepheno", "allpheno", "allpeaks"),
 print.summary.scanone <-
 function(x, ...)
 {
-  if(nrow(x) == 0) {
+  tab <- attr(x, "tab")
+
+  if(is.null(tab) && nrow(x) == 0) {
     cat("    There were no LOD peaks above the threshold.\n")
     return(invisible(NULL))
   }
@@ -393,7 +544,35 @@ function(x, ...)
     cat("\n")
   }
   
-  print.data.frame(x,digits=3)
+  flag <- FALSE
+  if(is.null(tab)) {
+    print.data.frame(x,digits=3)
+    flag <- TRUE
+  }
+  else if(tab=="tabByChr") {
+    for(i in seq(along=x)) {
+      if(is.null(x[[i]])) next
+      else {
+        flag <- TRUE
+        cat("Chr ", names(x)[i], ":\n", sep="")
+        print(x[[i]], digits=3)
+        cat("\n")
+      }
+    }
+  }
+  else if(tab=="tabByCol") {
+    for(i in seq(along=x)) {
+      if(nrow(x[[i]])==0) next
+      else {
+        flag <- TRUE
+        if(length(x) > 1) cat(names(x)[i], ":\n", sep="")
+        print(x[[i]], digits=3)
+        if(length(x) > 1) cat("\n")
+      }
+    }
+  }
+  if(!flag) 
+    cat("    There were no LOD peaks above the threshold.\n")
 }
 
 # pull out maximum LOD peak, genome-wide
@@ -483,6 +662,9 @@ c.scanone <-
 function(..., labels)
 {
   dots <- list(...)
+  cl1 <- class(dots[[1]])
+  if(length(dots)==1 && length(cl1)==1 && cl1=="list") dots <- dots[[1]]
+
   if(length(dots)==1) return(dots[[1]])
   for(i in seq(along=dots)) {
     if(!any(class(dots[[i]]) == "scanone"))
@@ -574,7 +756,7 @@ function(...)
 # scanone permutation test (from scanone with n.perm > 0)
 ######################################################################
 summary.scanoneperm <-
-function(object, alpha=c(0.05, 0.10), df=FALSE, ...)
+function(object, alpha=c(0.05, 0.10), df=FALSE, controlAcrossCol=FALSE, ...)
 {
   if(!any(class(object) == "scanoneperm"))
     stop("Input should have class \"scanoneperm\".")
@@ -584,28 +766,45 @@ function(object, alpha=c(0.05, 0.10), df=FALSE, ...)
 
   if("xchr" %in% names(attributes(object))) { # X-chromosome-specific results
     L <- attr(object, "L")
-    alphaA <- 1 - (1-alpha)^(L[1]/sum(L))
-    alphaX <- 1 - (1-alpha)^(L[2]/sum(L))
+    thealpha <- cbind(1 - (1-alpha)^(L[1]/sum(L)), 1 - (1-alpha)^(L[2]/sum(L)))
+    v <- c("A","X")
     
-    if(!is.matrix(object$A)) object$A <- as.matrix(object$A)
-    quA <- apply(object$A, 2, quantile, 1-alphaA, na.rm=TRUE)
-    if(!is.matrix(quA)) {
-      nam <- names(quA)
-      quA <- matrix(quA, nrow=length(alpha))
-      dimnames(quA) <- list(paste(100*alpha,"%", sep=""), nam)
-    }
-    else rownames(quA) <- paste(100*alpha, "%", sep="")
-    
-    if(!is.matrix(object$X)) object$X <- as.matrix(object$X)
-    quX <- apply(object$X, 2, quantile, 1-alphaX, na.rm=TRUE)
-    if(!is.matrix(quX)) {
-      nam <- names(quX)
-      quX <- matrix(quX, nrow=length(alpha))
-      dimnames(quX) <- list(paste(100*alpha,"%", sep=""), nam)
-    }
-    else rownames(quX) <- paste(100*alpha, "%", sep="")
+    quant <- vector("list", 2)
+    names(quant) <- c("A","X")
+    for(k in 1:2) {
+      if(!is.matrix(object[[v[k]]])) object[[v[k]]] <- as.matrix(object[[v[k]]])
 
-    quant <- list("A"=quA, "X"=quX)
+      if(controlAcrossCol) {
+        if(any(is.na(object[[v[k]]])))
+          object[[v[k]]] <- object[[v[k]]][apply(object[[v[k]]],1,function(a) !any(is.na(a))),,drop=FALSE]
+
+        r <- apply(object[[v[k]]], 2, rank, ties.method="random", na.last=FALSE)
+        print(is.matrix(r))
+        rmax <- apply(r, 1, max)
+        rqu <- quantile(rmax, 1-thealpha[,k], na.rm=TRUE)
+        qu <- matrix(nrow=length(thealpha[,k]), ncol=ncol(object[[v[k]]]))
+        object.sort <- apply(object[[v[k]]], 2, sort, na.last=FALSE)
+
+        for(i in seq(along=rqu)) {
+          if(fl==ce) # exact
+            qu[i,] <- object.sort[rqu[i],]
+          else # need to interpolate
+            qu[i,] <- object.sort[fl,]*(1-(ce-fl)) + object.sort[ce,]*(ce-fl)
+        }
+        colnames(qu) <- colnames(object[[v[k]]])
+      }
+      else 
+        qu <- apply(object[[v[k]]], 2, quantile, 1-thealpha[,k], na.rm=TRUE)
+      
+      if(!is.matrix(qu)) {
+        nam <- names(qu)
+        qu <- matrix(qu, nrow=length(alpha))
+        dimnames(qu) <- list(paste(100*alpha,"%", sep=""), nam)
+      }
+      else rownames(qu) <- paste(100*alpha, "%", sep="")
+
+      quant[[k]] <- qu
+    }
 
     if(df && "df" %in% names(attributes(object)))
       attr(quant,"df") <- attr(object, "df")
@@ -615,7 +814,29 @@ function(object, alpha=c(0.05, 0.10), df=FALSE, ...)
   }
   else {
     if(!is.matrix(object)) object <- as.matrix(object)
-    quant <- apply(object, 2, quantile, 1-alpha, na.rm=TRUE)
+
+    if(controlAcrossCol) {
+      if(any(is.na(object)))
+          object <- object[apply(object,1,function(a) !any(is.na(a))),,drop=FALSE]
+
+      r <- apply(object, 2, rank, ties.method="random", na.last=FALSE)
+      rmax <- apply(r, 1, max)
+      rqu <- quantile(rmax, 1-alpha, na.rm=TRUE)
+      quant <- matrix(nrow=length(alpha), ncol=ncol(object))
+      object.sort <- apply(object, 2, sort, na.last=FALSE)
+      for(i in seq(along=rqu)) {
+        fl <- floor(rqu[i])
+        ce <- ceiling(rqu[i])
+        if(fl==ce) # exact
+          quant[i,] <- object.sort[rqu[i],]
+        else # need to interpolate
+          quant[i,] <- object.sort[fl,]*(1-(ce-fl)) + object.sort[ce,]*(ce-fl)
+      }
+      colnames(quant) <- colnames(object)
+    }
+    else 
+      quant <- apply(object, 2, quantile, 1-alpha, na.rm=TRUE)
+
     if(!is.matrix(quant)) {
       nam <- names(quant)
       quant <- matrix(quant, nrow=length(alpha))
@@ -673,6 +894,10 @@ rbind.scanoneperm <- c.scanoneperm <-
 function(...)
 {
   dots <- list(...)
+
+  cl1 <- class(dots[[1]])
+  if(length(dots)==1 && length(cl1)==1 && cl1=="list") dots <- dots[[1]]
+
   if(length(dots)==1) return(dots[[1]])
   for(i in seq(along=dots)) {
     if(!any(class(dots[[i]]) == "scanoneperm"))
@@ -862,35 +1087,53 @@ function(..., labels)
 # subset.scanoneperm: pull out a set of lodcolumns
 ##############################
 subset.scanoneperm <-
-function(x, lodcolumn=1, ...)
+function(x, repl, lodcolumn, ...)
 {
+  att <- attributes(x)
+
   if(is.list(x)) {
-    if(is.matrix(x[[1]]) & ncol(x[[1]]) > 1) {
-      if((is.logical(lodcolumn) && length(lodcolumn) != ncol(x[[1]])) ||
-         (!is.logical(lodcolumn) && ((any(lodcolumn > 0) && any(lodcolumn > ncol(x[[1]]) | lodcolumn < 1)) ||
-         (any(lodcolumn < 0) && any(lodcolumn < -ncol(x[[1]])) | lodcolumn > -1))))
-        stop("lodcolumn misspecified.")
-      cl <- class(x)
-      x <- lapply(x, function(a,b) a[,b,drop=FALSE], lodcolumn)
-      class(x) <- cl
-    }
-    else stop("No need to subset; just one column.")
+    if(any(!sapply(x, is.matrix)))
+      x <- lapply(x, as.matrix)
+                  
+    if(missing(lodcolumn)) lodcolumn <- 1:ncol(x[[1]])
+    else if(!is.null(attr(try(x[[1]][,lodcolumn], silent=TRUE),"try-error")))
+      stop("lodcolumn misspecified.")
+
+    if(missing(repl)) repl <- 1:nrow(x[[1]])
+    else if(!is.null(attr(try(x[[1]][repl,], silent=TRUE),"try-error")))
+      stop("repl misspecified.")
+
+    cl <- class(x)
+    x <- lapply(x, function(a,b,d) unclass(a)[b,d,drop=FALSE], repl, lodcolumn)
+    class(x) <- cl
   }
   else {
-    if(is.matrix(x) & ncol(x > 1)) {
-      if((is.logical(lodcolumn) && length(lodcolumn) != ncol(x)) ||
-         (!is.logical(lodcolumn) && ((any(lodcolumn > 0) && any(lodcolumn > ncol(x) | lodcolumn < 1)) ||
-         (any(lodcolumn < 0) && any(lodcolumn < -ncol(x) | lodcolumn > -1)))))
-        stop("lodcolumn misspecified.")
-      cl <- class(x)
-      x <- x[,lodcolumn,drop=FALSE]
-      class(x) <- cl
-    }
-    else stop("No need to subset; just one column.")
+    if(!is.matrix(x)) x <- as.matrix(x)
+    
+    if(missing(lodcolumn)) lodcolumn <- 1:ncol(x)
+    else if(!is.null(attr(try(x[,lodcolumn], silent=TRUE),"try-error")))
+      stop("lodcolumn misspecified.")
+
+    if(missing(repl)) repl <- 1:nrow(x)
+    else if(!is.null(attr(try(x[repl,], silent=TRUE),"try-error")))
+      stop("repl misspecified.")
+    
+    cl <- class(x)
+    x <- unclass(x)[repl,lodcolumn,drop=FALSE]
+    class(x) <- cl
   }
+
+  for(i in seq(along=att)) {
+    if(names(att)[i] == "dim" || length(grep("names", names(att)[i]))>0) next
+    attr(x, names(att)[i]) <- att[[i]]
+  }
+
   x
 }
 
-
+# subset.scanoneperm using [,]
+`[.scanoneperm` <-
+function(x, repl, lodcolumn)
+  subset.scanoneperm(x, repl, lodcolumn)
 
 # end of summary.scanone.R
