@@ -7,7 +7,7 @@
  * This file written mostly by Karl Broman with some additions
  * from Hao Wu.
  *
- * last modified Feb, 2010
+ * last modified Nov, 2010
  * first written Feb, 2001
  *
  *     This program is free software; you can redistribute it and/or
@@ -35,7 +35,8 @@
  *                  wtaverage, comparegeno, R_comparegeno
  *                  R_locate_xo, locate_xo, matmult, expand_col2drop
  *                  dropcol_xpx, dropcol_xpy, dropcol_x,
- *                  reviseMWril, R_reviseMWril
+ *                  reviseMWril, R_reviseMWril, R_calcPermPval,
+ *                  calcPermPval
  *
  **********************************************************************/
 
@@ -294,13 +295,14 @@ void R_locate_xo(int *n_ind, int *n_mar, int *type,
 		 int *geno, double *map, 
 		 double *location, int *nseen,
 		 int *ileft, int *iright, double *left, double *right,
-		 int *full_info);
+		 int *ntyped, int *full_info);
 
 /* Note: type ==0 for backcross and ==1 for intercross */
 void locate_xo(int n_ind, int n_mar, int type, int **Geno,
 	       double *map, double **Location, 
 	       int *nseen, int **iLeft, int **iRight,
-	       double **Left, double **Right, int full_info);
+	       double **Left, double **Right, int **nTyped, 
+	       int full_info);
 
 /* multiply two matrices - I'm using dgemm from lapack here */
 void matmult(double *result, double *a, int nrowa,
@@ -355,6 +357,13 @@ void R_reviseMWril(int *n_ril, int *n_mar, int *n_str,
                    int *parents, int *geno, int *crosses,
                    int *missingval);
 
+/* wrapper for calcPermPval */
+void R_calcPermPval(double *peaks, int *nc_peaks, int *nr_peaks,
+		    double *perms, int *n_perms, double *pval);
+
+/* calculate permutation p-values for summary.scanone() */
+void calcPermPval(double **Peaks, int nc_peaks, int nr_peaks,
+		  double **Perms, int n_perms, double **Pval);
 #ifdef __cplusplus
   }
 #endif
